/*
** Module   : ClientBuilder.java
** Abstract : build and start a platform process.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131118 First version.
** 003 MAG 20140206 Send the referrer address via command line.
*/

package com.goldencode.p2j.main;

import java.io.*;
import java.util.*;
import java.util.logging.*;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.net.*;
import com.goldencode.p2j.util.*;

/**
 * This class is used to create and spawn a web client process using a
 * <code>ProcessBuilder</code> instance. A special small tools are designed for
 * Linux / Windows OS in order to spawn the process on the OS user account.
 * Both tools are written in C using specific OS API and are build as executable
 * using ant native target. The source code and the make file for this tools are 
 * available in the /src/native process directory. 
 * The password is provided vis stdin redirection on both platforms. 
 * The command line parameters for the spawned process are constructed from default 
 * parameters stored in directory and some specific parameters stored inside a 
 * <code>ClientBuilderParameters</code> structure.   
 * 
 * @author mag
 *
 */
public class ClientBuilder
{   
   /** Logger */
   private static final Logger LOG = LogHelper.getLogger(ClientBuilder.class.getName());

   /** The ProcessBuilder instance */
   private ProcessBuilder pb = new ProcessBuilder();            

   /** A collection of command line tokens */
   private List<String> command = new LinkedList<String>();
      
   /** A Java Bean containing specific parameters */
   private ClientBuilderParameters  config;   
   
   /** Server key store */
   private ServerKeyStore serverKeyStore;
   
   /**
    * Constructor.
    * 
    * @param   config
    *          The specific parameters for this client.
    */
   public ClientBuilder(ClientBuilderParameters  config)
   {
      this.config = config;
   }
 
   /**
    * Get server key store
    * 
    * @return  The loaded server key store.
    */
   public ServerKeyStore getServerKeyStore()
   {
      return (serverKeyStore != null && serverKeyStore.isValid()) ? serverKeyStore : null;
   }

   /**
    * Add terminal options.
    *  
    */
   private void addTerminalOptions()
   {
      Map<String, String> terminalOptions = config.getTerminalOptions();
      // add to command
      command.add("client:driver:type=" + terminalOptions.get("type"));
      command.add("client:chui:rows=" + terminalOptions.get("rows"));
      command.add("client:chui:columns=" + terminalOptions.get("columns"));
      command.add("client:chui:background=" + terminalOptions.get("background"));
      command.add("client:chui:foreground=" + terminalOptions.get("foreground"));
      command.add("client:chui:fontname=" + terminalOptions.get("fontname"));
      command.add("client:chui:fontsize=" + terminalOptions.get("fontsize"));
   }

   /**
    * Add server properties.
    * 
    * @param   secure
    *          Indicating to setup a secure <code>true</true> 
    *          or an insecure <code>false</code> connection.
    */
   private void addServerOptions(boolean secure)
   {
      // uuid assigned to this client
      command.add("server:spawner:uuid=" + config.getUuid());
      
      // add target server ports
      BootstrapConfig serverConfig = SessionManager.get().config();
      // load server key store
      serverKeyStore = new ServerKeyStore(serverConfig);

      String host = serverConfig.getString("net", "server", "host", "localhost");
      command.add("net:server:host=" + host);

      if (secure)
      {
         command.add("net:connection:secure=true");
         getServerConfig(serverConfig, "net", "server", "secure_port", -1);
      }
      else if (getServerConfig(serverConfig, "net", "server", "port", -1) == -1)
      {
         getServerConfig(serverConfig, "net", "server", "insecure_port", -1);
      }
   }

   /**
    * Get a key value form configuration file. 
    * If key exists add the value to builder command.
    * 
    * @param   config
    *          Configuration file.
    * @param   cname
    *          Category name.
    * @param   gname
    *          Group name.
    * @param   kname
    *          Key name.
    * @param   def
    *          Key value if exists default value otherwise.
    * 
    * @return  The integer value for key.
    */
   private int getServerConfig(BootstrapConfig config,
                               String cname, 
                               String gname, 
                               String kname,
                               int def)
   {
      int value = config.getInt(cname, gname, kname, def);
      if (value != def)
      {
         command.add(String.format("%s:%s:%s=%d", cname, gname, kname, value));
      }
      return value;
   }

   /**
    * Add Java CLASSPATH and the main class.
    * 
    */
   private void addClasspath()
   {
      Class<?> mainClass = config.getMainClass();
      // Native libraries path
      String libPath = config.getLibPath();
      command.add(String.format("-Djava.library.path=%s", libPath));
      // Add CLASSPATH
      command.add("-classpath");
      command.add(config.getClassPath());
      // Add main class
      command.add(mainClass.getName());
   }

   /**
    * Add client configuration file if exists.
    * 
    * @param   userDir
    *          Client working directory.
    */
   private void addConfigFile(String userDir)
   {
      String configFile = config.getConfigFile();
      if (configFile != null)
      {
         String clientConfig = String.format("%s%s%s", userDir, File.separator, configFile);
         if (new File(clientConfig).exists())
         {
            command.add(configFile);
         } 
         else
         {
            LOG.logp(Level.WARNING, 
                     "ClientBuilder.addConfigFile()", 
                     "", 
                     LogHelper.generate("%s does not exist.", clientConfig));
         }
      }
   }

   /**
    * Sets this process builder's working directory.
    * 
    * @param   userDir
    *          Server working directory.
    */
   @SuppressWarnings("unused")
   private File setUserDir(String userDir)
   {
      File fUserDir = new File(userDir);
      if (!fUserDir.exists())
      {
         String msg = "The user dir '%s' specified does not exist.";
         throw new IllegalStateException(String.format(msg, userDir));
      }
      return fUserDir;
   }

   /**
    * Spawn s new Java process.
    * 
    * @param   env
    *          A map with environment variables for temporary account.
    *           
    * @return  Process object.
    * 
    * @throws  IOException
    *          If something goes wrong.
    */
   public Process start(Map<String, String> env) throws IOException
   {
      // Set temporary account environment variables
      Map<String, String> environment = pb.environment();
      environment.putAll(env);      
      // Get working directory
      String userDir = config.getWorkingDirectory();
      pb.directory(null);
      // Add OS spawner
      command.add(config.getSpawner());
      // User name.
      command.add(config.getUserName());
      // Working directory
      command.add(userDir);
      // Build command
      command.add(config.getCommand());
      // Add JVM arguments
      command.addAll(config.getJvmArguments());
      // Add CLASSPATH and main class
      addClasspath();
      // Add client configuration file
      addConfigFile(userDir);
      // Add server options
      addServerOptions(config.isSecure());
      // Add terminal options
      addTerminalOptions();
      // Add referrer address
      command.add("web:referrer:url=" + config.getReferrer());
      pb.command(command);
      Process shell = pb.start();
      // Write password
      OutputStreamWriter os = new OutputStreamWriter(shell.getOutputStream());
      os.write(config.getPassword());
      os.close();     
      return shell;
   }
}
