/*
** Module   : WebClientSpawner.java
** Abstract : define the Web Client Spawner.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131119 First version.
** 002 MAG 20140206 Send URL as referrer address via configuration file.
*/

package com.goldencode.p2j.main;

import java.util.concurrent.*;
import java.util.logging.*;

import com.goldencode.p2j.util.*;

/**
 * Class used to spawn a new web client process.
 * The spawned process first try to authenticate the user against OS.
 * On success a new process (client) is spawned. The spawned process will get the 
 * server KeyStore, starts the client embedded server and finally will notify this 
 * instance about the URI on which the client embedded server listen.
 * If a notification does not occur on an amount of time a failure in spawned 
 * process is asserted.  
 * 
 * @author mag
 *
 */
public class WebClientSpawner
implements SpawnerListener
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(WebClientSpawner.class.getName());
   
   /** Timeout */
   private static final int TIME_OUT = 30;
   
   /** Remote server URI */
   private volatile String remoteUri = null;
   
   /** Client Builder */
   private ClientBuilder cb;
   
   /** Spawner exit code */
   private int exitCode = 0;
   
   /** Synchronize */
   private CountDownLatch latch;
   
   /**
    * Launch a Web Server Client
    * 
    * @param   userName
    *          User name.
    * @param   password
    *          User password. 
    * @param   referrer
    *          Back link URL. 
    * 
    * @return  The remote server URI or <code>null</code>
    */
   public String spawn(String userName, String password, String referrer)
   {
      // Get a temporary account from pool.
      TemporaryAccount tmpAcc = TemporaryAccount.open();
      if (tmpAcc == null)
      {
         String message = "Temporary Account Pool is empty! " +
         		"Please call your system administrator.";
         LOG.logp(Level.WARNING, 
                  "WebHandler.spawn()", 
                  "",
                  LogHelper.generate(message));
         
         throw new RuntimeException(message);
      }
      // Client builder
      ClientBuilderParameters cfg = new ClientBuilderParameters();
      cfg.setUserName(userName);
      cfg.setPassword(password);
      cfg.setReferrer(referrer);

      cb = new ClientBuilder(cfg);
      // register listener
      SpawnerImpl spawner = (SpawnerImpl) SpawnerImpl.getInstance();
      spawner.addListener(cfg.getUuid(), this);
      
      // spawn     
      Process shell = null;
      try
      {
         latch = new CountDownLatch(1);

         // spawn 
         shell = cb.start(tmpAcc.getEnvironmentMap());        
         
         // wait for notification
         latch.await(TIME_OUT, TimeUnit.SECONDS);
         
         shell.waitFor();
         exitCode = shell.exitValue();
      }
      catch (Exception e)
      {
         LOG.logp(Level.SEVERE, 
                  "WebHandler.spawn()", 
                  "",
                  LogHelper.generate("Cannot spawn the process."),
                  e);
      }
      finally
      {
         spawner.removeListener(cfg.getUuid());
         tmpAcc.close();
      }
      
      if (remoteUri == null)
      {
         shell.destroy();
      }
      
      return remoteUri;
   }

   /**
    * Return spawner exit code
    * 
    * @return  Spawn exit value.
    */
   public int getExitCode()
   {
      return exitCode;
   }

   /**
    * Notification method on remote server running.
    * 
    * @param   uri
    *          Remote server URI.
    */
   @Override
   public void embeddedWebClientIsReady(String uri)
   {
      remoteUri = uri;
      latch.countDown();
   }

   /**
    * Export server key store
    */
   @Override
   public ServerKeyStore getServerKeyStore()
   {
      return cb.getServerKeyStore();
   }   
}
