/*
** Module   : WatchdogTimer.java
** Abstract : a watchdog timer implementation.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140131 Created initial version.
*/

package com.goldencode.p2j.ui.client.chui.driver.web;

/**
 * A watchdog timer thread implementation.
 * The watchdog timer is used to guard the web client life.
 * When the web socket is not connected the watchdog runs.
 * On watchdog thread if timeout elapsed than the process exit.  
 * When the web socket is connected the watchdog is not active. 
 *
 */
public class WatchdogTimer
extends Thread
{
   /** time to wait for thread exit */
   public static final long WAIT_TO_DIE = 5000;

   /** one minutes as microseconds */
   private static final long ONE_MINUTE = 60000;
   
   /** time when started */
   private volatile long startTime;

   /** time out value in milliseconds */
   private volatile long timeout;
   
   /** loop condition */
   private volatile boolean running = true;
   
   /**
    * Create a new instance and start the thread.
    * 
    * @param   timeout
    *          time out value in milliseconds.
    */
   public WatchdogTimer(long timeout)
   {
      this.setName("watchdog");
      this.timeout = timeout;
      this.start();
   }

   /**
    * Because the Thread.sleep time is not guaranteed to be accuracy the elapsed
    * time is calculated every minutes. If elapsed time is greater than the timeout 
    * value a process exit is forced. Also the spurious wake-up is avoided using 
    * a loop condition.
    */
   @Override
   public void run()
   {
      // start time
      startTime = System.currentTimeMillis();
      
      while (running)
      {
         try
         {
            Thread.sleep(WatchdogTimer.ONE_MINUTE);
            // every minutes check elapsed time and force an exit on timeout
            long elapsedTime = System.currentTimeMillis() - startTime;            
            if (running && elapsedTime >= timeout)
            {
               System.exit(-1);
            }
         }
         catch (InterruptedException e)
         {
         }
      }
   }
   
   /**
    * Terminate watchdog timer.
    */
   public void kill()
   {
      running = false;
      this.interrupt();
   }   
}
